@@ -1,7 +1,7 @@
 /*
  * date.h -- functions for dealing with dates.
  *
- * Copyright (C)1999-2006 Mark Simpson <damned@theworld.com>
+ * Copyright (C)1999-2018 Mark Simpson <damned@theworld.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,13 +31,12 @@
 /* Store a date according to file specification */
 struct date
 {
-    int16 year, month, day;
-    int16 hour, min, sec;
-    int16 dow;
+    uint16 year, month, day;
+    uint16 hour, min, sec;
+    uint16 dow;
 };
 
 extern const char *date_to_str (struct date* dt);
 extern void date_read (struct date *dt, const unsigned char *buf);
 
 #endif /* DATE_H */
-
